PREFIX = 0x0C
PREFIX_RESP = b"\xB1\xCB\x74\x00"

TYPE_CONNECT = 0x000D
TYPE_HEART = 0x0004
TYPE_GBBQ = 0x000F
TYPE_COUNT = 0x044E
TYPE_CODE = 0x0450
TYPE_MINUTE = 0x051D
TYPE_KLINE = 0x052D
TYPE_QUOTE = 0x053E
TYPE_CALL_AUCTION = 0x056A
TYPE_HISTORY_MINUTE = 0x0FB4
TYPE_HISTORY_TRADE = 0x0FB5
TYPE_TRADE = 0x0FC5

CONTROL_DEFAULT = 0x01
CODE_PAGE_SIZE = 1000
KLINE_PAGE_SIZE = 800
TRADE_PAGE_SIZE = 1800
HISTORY_TRADE_PAGE_SIZE = 2000
